@@ -17,19 +17,19 @@
 #
 #############################################################################
 #'
-#' Fuzzification refers to the process of transforming a crisp set into fuzzy terms. 
+#' Fuzzification refers to the process of transforming a crisp set into linguistic terms. 
 #'
 #' In this function, there are five shapes of membership functions implemented, 
 #' namely \code{TRIANGLE}, \code{TRAPEZOID}, \code{GAUSSIAN}, \code{SIGMOID}, and \code{BELL}.
 #' 
-#' @title Transform from crisp set into fuzzy terms
+#' @title Transform from crisp set into linguistic terms
 #'
 #' @param data a matrix of data containing numerical elements.
 #' @param num.varinput number of input variables.
 #' @param num.labels.input the number of labels of the input variables.
 #' @param varinp.mf a matrix containing the parameters to form the membership functions. 
-#' The dimension of the matrix is (5, n) where n is 
-#' a multiplication the number of fuzzy terms/labels and the number of input variables.
+#' The dimension of the matrix is (\eqn{5, n}) where \eqn{n} is 
+#' a multiplication the number of linguistic terms/labels and the number of input variables.
 #' The rows of the matrix represent:
 #' The first row is the type of membership function, where 1 means \code{TRIANGLE}, 
 #' 2 means \code{TRAPEZOID} in left side, 
@@ -38,12 +38,12 @@
 #' 6 means \code{SIGMOID}, and 7 means \code{BELL}. And, the second up to fifth row indicate 
 #' the corner points to construct the functions. 
 #' \itemize{
-#' \item \code{TRIANGLE} has three parameters (a, b, c), where b is the center point of the TRIANGLE,
-#' and a and c are the left and right points, respectively.
-#' \item \code{TRAPEZOID} has four parameters (a, b, c, d).
-#' \item \code{GAUSSIAN} has two parameters (mean and variance).
-#' \item \code{SIGMOID} has two parameters (gamma and c).
-#' \item \code{BELL} has three parameters (a, b, c).
+#' \item \code{TRIANGLE} has three parameters (\eqn{a, b, c}), where \eqn{b} is the center point of the \code{TRIANGLE},
+#' and \eqn{a} and \eqn{c} are the left and right points, respectively.
+#' \item \code{TRAPEZOID} has four parameters (\eqn{a, b, c, d}).
+#' \item \code{GAUSSIAN} has two parameters (\eqn{mean} and \eqn{variance}).
+#' \item \code{SIGMOID} has two parameters (\eqn{\gamma} and \eqn{c}).
+#' \item \code{BELL} has three parameters (\eqn{a, b, c}).
 #' }
 #' 
 #' For example:
@@ -53,7 +53,7 @@
 #' \code{30,80,40,70,100,60,100,0,0,100,0,100), nrow=5, byrow=TRUE)}
 #'
 #' @seealso \code{\link{defuzzifier}}, \code{\link{rulebase}}, and \code{\link{inference}}
-#' @return A matrix of the degree of each fuzzy terms based on the shape of 
+#' @return A matrix of the degree of each linguistic terms based on the shape of 
 #' the membership functions 
 #' @export
 fuzzifier <- function(data, num.varinput, num.labels.input, varinp.mf){
@@ -98,14 +98,10 @@ for (h in 1 : nrow(data)){
 			##b=varinp.mf[3,]
 			##c=varinp.mf[4,]
 			if (varinp.mf[1, j] == 1){
-				if (data[h, i] <= varinp.mf[2, j])
-				temp <- 0
-				else if (data[h, i] <= varinp.mf[3, j])
-				temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
-				else if (data[h, i] < varinp.mf[4, j])
-				temp <- (data[h, i] - varinp.mf[4, j]) / (varinp.mf[3, j] - varinp.mf[4, j])
-				else
-				temp <- 0
+				if (data[h, i] <= varinp.mf[2, j]) temp <- 0
+				else if (data[h, i] <= varinp.mf[3, j]) temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
+				else if (data[h, i] < varinp.mf[4, j]) temp <- (data[h, i] - varinp.mf[4, j]) / (varinp.mf[3, j] - varinp.mf[4, j])
+				else temp <- 0
 			}
 
 			##checking for type 2: Trapezoid_1a, if varinp.mf[1,] ==2
@@ -115,12 +111,9 @@ for (h in 1 : nrow(data)){
 			##b=varinp.mf[3,]
 			##c=varinp.mf[4,]
 			else if (varinp.mf[1, j] == 2){
-				if (data[h, i] <= varinp.mf[3, j])
-				temp <- 1
-				else if (data[h, i] <= varinp.mf[4, j])
-				temp <- (data[h, i] - varinp.mf[4, j]) / (varinp.mf[3, j] - varinp.mf[4, j])
-				else
-				temp <- 0				
+				if (data[h, i] <= varinp.mf[3, j]) temp <- 1
+				else if (data[h, i] <= varinp.mf[4, j]) temp <- (data[h, i] - varinp.mf[4, j]) / (varinp.mf[3, j] - varinp.mf[4, j])
+				else temp <- 0				
 			}
 
 			##checking for type 3: Trapezoid_1b, if varinp.mf[1,] == 3
@@ -130,12 +123,9 @@ for (h in 1 : nrow(data)){
 			##b=varinp.mf[3,]
 			##c=varinp.mf[4,]
 			else if (varinp.mf[1, j] == 3){
-				if (data[h, i] <= varinp.mf[2, j])
-				temp <- 0
-				else if (data[h, i] < varinp.mf[3, j])
-				temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
-				else 
-				temp <- 1
+				if (data[h, i] <= varinp.mf[2, j]) temp <- 0
+				else if (data[h, i] < varinp.mf[3, j]) temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
+				else temp <- 1
 			}
 
 			##checking for type 4: Trapezoid_2
@@ -145,14 +135,10 @@ for (h in 1 : nrow(data)){
 			##c=varinp.mf[4,]
 			##d=varinp.mf[5,]
 			else if (varinp.mf[1, j] == 4){
-				if (data[h,i] <= varinp.mf[2, j] || data[h,i] > varinp.mf[5, j])
-				temp <- 0
-				else if (data[h, i] > varinp.mf[3, j] && data[h, i] <= varinp.mf[4, j])
-				temp <- 1
-				else if (data[h, i] > varinp.mf[2, j] && data[h, i] <= varinp.mf[3, j])
-				temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
-				else if (data[h,i] > varinp.mf[4, j] && data[h,i] <= varinp.mf[5,j])
-				temp <- (data[h, i] - varinp.mf[5, j]) / (varinp.mf[4, j] - varinp.mf[5, j])
+				if (data[h,i] <= varinp.mf[2, j] || data[h,i] > varinp.mf[5, j]) temp <- 0
+				else if (data[h, i] > varinp.mf[3, j] && data[h, i] <= varinp.mf[4, j]) temp <- 1
+				else if (data[h, i] > varinp.mf[2, j] && data[h, i] <= varinp.mf[3, j]) temp <- (data[h, i] - varinp.mf[2, j]) / (varinp.mf[3, j] - varinp.mf[2, j])
+				else if (data[h,i] > varinp.mf[4, j] && data[h,i] <= varinp.mf[5,j]) temp <- (data[h, i] - varinp.mf[5, j]) / (varinp.mf[4, j] - varinp.mf[5, j])
 			}
 
 			##checking for type 5: Gaussian
@@ -180,12 +166,13 @@ for (h in 1 : nrow(data)){
 				temp <- 1/(1 + abs((data[h, i] - varinp.mf[4, j])/varinp.mf[2, ]) ^ (2 * varinp.mf[3, ]))   
 			}
 
-		##save membership function on MF for each data		
-		MF[h, j] <- temp
+			##save membership function on MF for each data		
+			MF[h, j] <- temp
 		
-		ii <- ii + 1
-		jj <- jj + 1
-		##this checking is used for control the number of fuzzy value for each variable
+			ii <- ii + 1
+			jj <- jj + 1
+			##this checking is used for control the number of linguistic value for each variable
+			
 			if (ii > num.labels.input[1, i])
 			break
 		
@@ -209,7 +196,7 @@ return(MF)
 #'
 #' It means that "IF input.variable1 is a1 and input.variable2 is b1 THEN output.variable is c1"
 #' 
-#' Here, ("a1", "and", "b1") is the antecedent, with "a1" and "b1" being fuzzy terms, 
+#' Here, ("a1", "and", "b1") is the antecedent, with "a1" and "b1" being linguistic terms, 
 #' and ("c1") is the consequent part. 
 #' 
 #' A fuzzy IF-THEN rule base with several rules is defined in the following way:
@@ -236,22 +223,22 @@ return(MF)
 #' The dimension of this matrix is [<number_of_rules>, <number_of_variables> + 1]. The matrix has one extra column for the constants. 
 #' If there is no constant, a zero is put.
 #'
-#' So, for example, if we have 3 rules and 2 fuzzy variables (A, B), the matrix \code{fun_tsk} has dim(3,3), as in: 
+#' So, for example, if we have 3 rules and 2 linguistic variables (A, B), the matrix \code{fun_tsk} has dim(3,3), as in: 
 #' 
 #' \code{func.tsk <- matrix(c(1, 1, 5, 2, 1, 3, 1, 2, 2), nrow=3, ncol=3, byrow = TRUE)}
 #'
 #' Furthermore, we can represent linguistic hedges within the rules. The kinds of hedges used are
 #'
 #' \itemize{
-#' \item "extremely" reduces the truth value. For example, "extremely a1" means membership function a1 = miu(a1)^3 
+#' \item \code{"extremely"} reduces the truth value. For example, \code{"extremely a1"} means membership function \eqn{a1 = \mu(a1)^3}.
 #'
-#' \item "very" reduces the truth value. For example, "very a1" means membership function a1 = miu(a1)^2
+#' \item \code{"very"} reduces the truth value. For example, \code{"very a1"} means membership function \eqn{a1 = \mu(a1)^2}.
 #'
-#' \item "somewhat" increases the truth value. For example, "somewhat a1" means membership function 
-#' a1 = miu(a1)^0.5
+#' \item \code{"somewhat"} increases the truth value. For example, \code{"somewhat a1"} means membership function 
+#' \eqn{a1 = \mu(a1)^0.5}.
 #'
-#' \item "slightly" increases the truth value. For example, "slightly a1" means membership function 
-#' a1 = miu(a1)^0.33
+#' \item \code{"slightly"} increases the truth value. For example, \code{"slightly a1"} means membership function 
+#' \eqn{a1 = \mu(a1)^0.33}
 #' }
 #' 
 #' An example of fuzzy IF-THEN rules using linguistic hedge is: 
@@ -264,7 +251,7 @@ return(MF)
 #'
 #' \code{rule <- list(r1,r2,r3)} 
 #' 
-#' Furthermore, the following is an example in order to give names to the fuzzy terms in the input and output variables.
+#' Furthermore, the following is an example in order to give names to the linguistic terms in the input and output variables.
 #'
 #' \code{varinput.1 <- c("a1", "a2", "a3")}
 #'
@@ -286,9 +273,9 @@ return(MF)
 #'
 #'               \code{nrow=3, byrow=TRUE)} 
 #' 
-#' Where, "1", "2", "3" represent class 1, 2, and 3.
+#' Where, \code{"1"}, \code{"2"}, \code{"3"} represent class \code{1}, \code{2}, and \code{3}.
 #'
-#' Note that the names of the fuzzy terms must be unique and
+#' Note that the names of the linguistic terms must be unique and
 #' if we are using the learning methods, the fuzzy IF-THEN rules will be generated automatically 
 #' as the outputs of \code{\link{frbs.learn}}.
 #'
@@ -312,7 +299,7 @@ if (class(rule) == "matrix"){
 
 
 ##condition for Mamdani model
-if (type.model == 1 || type.model == "MAMDANI") {
+if (type.model == 1 || type.model == 2 || type.model == "MAMDANI" || type.model == "FRBCS" || type.model == "TSK") {
 	##Checking the sign of separating between antecedent and consequence 
 	for (i in 1 : length(rule)){
 		equal <- unlist(rule[i])
@@ -324,18 +311,9 @@ if (type.model == 1 || type.model == "MAMDANI") {
 }	
 
 #condition for Takagi Sugeno Kang model
-if (type.model == 2 || type.model == "TSK" || type.model == "FRBCS") {
-	
+if (type.model == 2 || type.model == "TSK") {	
 	if (is.null(func.tsk))
 		stop("You are using Takagi Sugeno Kang model, so the consequent part must be linear equations, please insert their values")
-	##Checking the sign of separating between antecedent and consequence 
-	for (i in 1 : length(rule)){
-		equal <- unlist(rule[i])
-		equal <- as.matrix(equal)
-		check <- any(equal == "->")
-		if (check == FALSE )
-			stop("rule must contain \"->\" as separating between antecendent and consequence") 				
-	}
 }
 
 return(rule)
@@ -347,26 +325,26 @@ return(rule)
 #' Furthermore, there are some formula for conjunction and disjunction operators. 
 #' 
 #' \bold{The Mamdani model:}
-#' A fuzzy system with, e.g., two inputs x1 and x2 (antecedents) and a single output y (consequent)
+#' A fuzzy system with, e.g., two inputs \eqn{x1} and \eqn{x2} (antecedents) and a single output \eqn{y} (consequent)
 #' is described by the following fuzzy IF-THEN rule:
 #'
 #' \code{IF x1 is A1 and x2 is A2 THEN y is B}
 #'
-#' where A1 and A2 are the fuzzy sets representing the antecent pairs and
-#' B is the fuzzy set representing the consequent.
+#' where \eqn{A1} and \eqn{A2} are the fuzzy sets representing the antecent pairs and
+#' \eqn{B} is the fuzzy set representing the consequent.
 #'
 #' \bold{The Takagi Sugeno Kang model:}
-#' Suppose we have two inputs x1 and x2 and output y, then the fuzzy IF-THEN rule is as follows:
+#' Suppose we have two inputs \eqn{x1} and \eqn{x2} and output \eqn{y}, then the fuzzy IF-THEN rule is as follows:
 #'
 #' \code{IF x1 is A1 and x2 is A2 THEN y is y = f(x1, x2)}
 #'
-#' where y = f(x1, x2) is a crisp function in the consequent part which is usually a polynomial function,
-#' and A1 and A2 are the fuzzy sets representing the antecent pairs.
+#' where \eqn{y = f(x1, x2)} is a crisp function in the consequent part which is usually a polynomial function,
+#' and \eqn{A1} and \eqn{A2} are the fuzzy sets representing the antecent pairs.
 #' 
 #' Futhermore, this function has the following capabilities: 
 #' \itemize{ 
-#' \item It supports unary operators (not) and binary operators (AND and OR).
-#' \item It provides linguistic hedge (extremely, very, somewhat, slightly).
+#' \item It supports unary operators (not) and binary operators (\code{AND} and \code{OR}).
+#' \item It provides linguistic hedge (\code{extremely}, \code{very}, \code{somewhat}, \code{slightly}).
 #' \item there are several methods for the t-norm and s-norm.
 #' }
 #' @title The process of fuzzy reasoning
@@ -376,19 +354,19 @@ return(rule)
 #' @param names.varinput a list of names of the input variables. 
 #' @param type.tnorm a value which represents the type of t-norm to be used: 
 #' \itemize{
-#' \item \code{1} or \code{MIN} means standard t-norm: min(x1, x2).
-#' \item \code{2} or \code{HAMACHER} means Hamacher product: (x1 * x2)/(x1 + x2 - x1 * x2).
-#' \item \code{3} or \code{YAGER} means Yager class (with tao = 1): 1- min(1, ((1 - x1) + (1 - x2))).
-#' \item \code{4} or \code{PRODUCT} means product: (x1 * x2).
-#' \item \code{5} or \code{BOUNDED} means bounded product: max(0, x1 + x2 - 1).
+#' \item \code{1} or \code{MIN} means standard t-norm: \eqn{min(x1, x2)}.
+#' \item \code{2} or \code{HAMACHER} means Hamacher product: \eqn{(x1 * x2)/(x1 + x2 - x1 * x2)}.
+#' \item \code{3} or \code{YAGER} means Yager class: \eqn{1- min(1, ((1 - x1) + (1 - x2)))}.
+#' \item \code{4} or \code{PRODUCT} means product: \eqn{(x1 * x2)}.
+#' \item \code{5} or \code{BOUNDED} means bounded product: \eqn{max(0, x1 + x2 - 1)}.
 #' }
 #' @param type.snorm a value which represents the type of s-norm to be used: 
 #' \itemize{
-#' \item \code{1} or \code{MAX} means standard s-norm: max(x1, x2).
-#' \item \code{2} or \code{HAMACHER} means Hamacher sum: (x1 + x2 - 2x1 * x2) / 1 - x1 * x2.
-#' \item \code{3} or \code{YAGER} means Yager class (with tao = 1): min(1, (x1 + x2)).
-#' \item \code{4} or \code{SUM} means sum: (x1 + x2 - x1 * x2).
-#' \item \code{5} or \code{BOUNDED} means bounded sum: min(1, x1 + x2).
+#' \item \code{1} or \code{MAX} means standard s-norm: \eqn{max(x1, x2)}.
+#' \item \code{2} or \code{HAMACHER} means Hamacher sum: \eqn{(x1 + x2 - 2x1 * x2) / 1 - x1 * x2}.
+#' \item \code{3} or \code{YAGER} means Yager class: \eqn{min(1, (x1 + x2))}.
+#' \item \code{4} or \code{SUM} means sum: \eqn{(x1 + x2 - x1 * x2)}.
+#' \item \code{5} or \code{BOUNDED} means bounded sum: \eqn{min(1, x1 + x2)}.
 #' }
 #' @seealso \code{\link{defuzzifier}}, \code{\link{rulebase}}, and \code{\link{fuzzifier}}.
 #' @return a matrix of the degrees of the rules. 
@@ -418,14 +396,16 @@ for(k in 1 : nMF){
 		##detect location of "->" sign as separation between antecedent and consequence
 		loc <- which(temp == "->")
 				
-			##Inisialization for calculating MF of antecendet
-			temp.fvalue <- strsplit(temp[1], split=" ")
-						
-			Mtemp.fvalue <- unlist(temp.fvalue)
-			
-			length.MtempValue <- length(Mtemp.fvalue)
-			
-			##check that the fuzzy value has unary operator ("not" and linguistic hedge)
+		##Inisialization for calculating MF of antecendet
+		temp.fvalue <- strsplit(temp[1], split=" ")
+		Mtemp.fvalue <- unlist(temp.fvalue)	
+		length.MtempValue <- length(Mtemp.fvalue)
+		
+		#if (Mtemp.fvalue[1] == "dont_care"){
+		if (grepl("dont_care", Mtemp.fvalue[1])){
+				val.antecedent <- 1
+		} else {	
+			##check that the linguistic value has unary operator ("not" and linguistic hedge)
 			if (length.MtempValue > 1){
 				if (Mtemp.fvalue[1] == "not"){
 					if (Mtemp.fvalue[2] == "extremely") {
@@ -443,82 +423,74 @@ for(k in 1 : nMF){
 					else if (Mtemp.fvalue[2] == "slightly"){
 						val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^0.333
 						val.antecedent <- 1 - val.antecedent.temp
-					}
-					else 
+					} else {
 						val.antecedent <- 1 - MF[k, Mtemp.fvalue[2]]
+					}
 				}
 				else {
 					if (Mtemp.fvalue[1] == "extremely")
-					val.antecedent <- (MF[k, Mtemp.fvalue[2]])^3
+						val.antecedent <- (MF[k, Mtemp.fvalue[2]])^3
 					else if (Mtemp.fvalue[1] == "very")
-					val.antecedent <- (MF[k, Mtemp.fvalue[2]])^2
+						val.antecedent <- (MF[k, Mtemp.fvalue[2]])^2
 					else if (Mtemp.fvalue[1] == "somewhat")
-					val.antecedent <- (MF[k, Mtemp.fvalue[2]])^0.5
+						val.antecedent <- (MF[k, Mtemp.fvalue[2]])^0.5
 					else if (Mtemp.fvalue[1] == "slightly")
-					val.antecedent <- (MF[k, Mtemp.fvalue[2]])^0.333
+						val.antecedent <- (MF[k, Mtemp.fvalue[2]])^0.333
 					else if (Mtemp.fvalue[1] == "not")
-					val.antecedent <- 1 - (MF[k, Mtemp.fvalue[2]])
+						val.antecedent <- 1 - (MF[k, Mtemp.fvalue[2]])
 				}			
-			} 
-			
-			else{
-				if (Mtemp.fvalue[1] == "don't_care"){
-					val.antecedent <- 1
-				}
-				else{
-					val.antecedent <- MF[k, temp[1]]
-				}
+			} else {
+				val.antecedent <- MF[k, temp[1]]	
 			}	
-			## iterate to calculate degree of MF until find "->" sign (equals to loc)
-			seqq <- seq(from = 1, to = loc, by = 2)
-			for (j in seqq) {
-			
+		}
+		
+		## iterate to calculate degree of MF until find "->" sign (equals to loc)
+		seqq <- seq(from = 1, to = loc, by = 2)
+		for (j in seqq) {
 			temp.split <- strsplit(temp[j + 2], split = " ")
 			Mtemp.fvalue <- unlist(temp.split)
 			length.MtempValue <- length(Mtemp.fvalue)
-			
-			if (length.MtempValue > 1) {
-				if (Mtemp.fvalue[1] == "not"){
-					if (Mtemp.fvalue[2] == "extremely") {
-						val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^3
-						val.antecedent.b <- 1 - val.antecedent.temp
-					}
-					else if (Mtemp.fvalue[2] == "very"){
-						val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^2
-						val.antecedent.b <- 1 - val.antecedent.temp
-					}
-					else if (Mtemp.fvalue[2] == "somewhat"){
-						val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^0.5
-						val.antecedent.b <- 1 - val.antecedent.temp
-					}
-					else if (Mtemp.fvalue[2] == "slightly"){
-						val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^0.333
-						val.antecedent.b <- 1 - val.antecedent.temp
-					}
-					else 
-						val.antecedent.b <- 1 - MF[k, Mtemp.fvalue[2]]
-				}
-				else {
-					if (Mtemp.fvalue[1] == "extremely")
-					val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^3
-					else if (Mtemp.fvalue[1] == "very")
-					val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^2
-					else if (Mtemp.fvalue[1] == "somewhat")
-					val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^0.5
-					else if (Mtemp.fvalue[1] == "slightly")
-					val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^0.333
-					else if (Mtemp.fvalue[1] == "not")
-					val.antecedent.b <- 1 - (MF[k, Mtemp.fvalue[2]])
-				}		
-			} 
-			else{
-				if (Mtemp.fvalue[1] == "don't_care"){
+			#if (Mtemp.fvalue[1] == "dont_care"){
+			if (grepl("dont_care", Mtemp.fvalue[1])){
 					val.antecedent.b <- 1
-				}
+			} else {
+				if (length.MtempValue > 1) {
+					if (Mtemp.fvalue[1] == "not"){
+						if (Mtemp.fvalue[2] == "extremely") {
+							val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^3
+							val.antecedent.b <- 1 - val.antecedent.temp
+						}
+						else if (Mtemp.fvalue[2] == "very"){
+							val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^2
+							val.antecedent.b <- 1 - val.antecedent.temp
+						}
+						else if (Mtemp.fvalue[2] == "somewhat"){
+							val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^0.5
+							val.antecedent.b <- 1 - val.antecedent.temp
+						}
+						else if (Mtemp.fvalue[2] == "slightly"){
+							val.antecedent.temp <- (MF[k, Mtemp.fvalue[3]])^0.333
+							val.antecedent.b <- 1 - val.antecedent.temp
+						}
+						else 
+							val.antecedent.b <- 1 - MF[k, Mtemp.fvalue[2]]
+					}
+					else {
+						if (Mtemp.fvalue[1] == "extremely")
+						val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^3
+						else if (Mtemp.fvalue[1] == "very")
+						val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^2
+						else if (Mtemp.fvalue[1] == "somewhat")
+						val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^0.5
+						else if (Mtemp.fvalue[1] == "slightly")
+						val.antecedent.b <- (MF[k, Mtemp.fvalue[2]])^0.333
+						else if (Mtemp.fvalue[1] == "not")
+						val.antecedent.b <- 1 - (MF[k, Mtemp.fvalue[2]])
+					}		
+				} 
 				else{
 					val.antecedent.b <- MF[k, temp[j + 2]]
 				}
-				
 			}
 			##condition for conjunction operator (AND)
 			if ((temp[j + 1] == "1") || (temp[j + 1] == "and")){
@@ -537,9 +509,9 @@ for(k in 1 : nMF){
 					temp.val.ante <- (1 - val.antecedent) + (1 - val.antecedent.b)
 					if (temp.val.ante <= 1){
 						val.antecedent <- temp.val.ante
+					} else {
+						val.antecedent <- 1
 					}
-					else
-					val.antecedent <- 1
 				}
 				
 				##condition for type.tnorm used is product
@@ -552,9 +524,9 @@ for(k in 1 : nMF){
 					temp.val.ante <- (val.antecedent * val.antecedent.b - 1)
 					if (temp.val.ante > 0){
 						val.antecedent <- temp.val.ante
+					} else {
+						val.antecedent <- 0
 					}
-					else
-					val.antecedent <- 0
 				}
 			}
 
@@ -577,9 +549,9 @@ for(k in 1 : nMF){
 						temp.val.ante <- (val.antecedent + val.antecedent.b)
 						if (temp.val.ante <= 1){
 							val.antecedent <- temp.val.ante
-						}
-						else
+						} else {
 							val.antecedent <- 1						
+						}
 					}
 					
 					##condition for type.snorm used is sum
@@ -592,9 +564,9 @@ for(k in 1 : nMF){
 						temp.val.ante <- (val.antecedent + val.antecedent.b)
 						if (temp.val.ante <= 1){
 							val.antecedent <- temp.val.ante
-						}
-						else
+						} else {
 							val.antecedent <- 1					
+						}
 					}				
 			}
 			
@@ -602,7 +574,7 @@ for(k in 1 : nMF){
 			##Checking 
 			if (j == loc - 1)
 				break	
-			}
+		}
 		
 		##save value of MF on each rule			
 		miu.rule[k, i] <- c(val.antecedent)		
@@ -614,12 +586,12 @@ for(k in 1 : nMF){
 return(miu.rule)
 }
 
-#' Defuzzification is a transformation that extracts the crisp values from the fuzzy terms. 
+#' Defuzzification is a transformation that extracts the crisp values from the linguistic terms. 
 #' 
 #' In this function, there exist two kinds of models which are based on the Mamdani and 
 #' Takagi Sugeno Kang model. 
-#' For the Mamdani model there are five methods for defuzzifying a fuzzy term A of a universe 
-#' of discourse Z. 
+#' For the Mamdani model there are five methods for defuzzifying a linguistic term \eqn{A} of a universe 
+#' of discourse \eqn{Z}. 
 #' They are as follows:
 #' \enumerate{
 #' \item weighted average method (\code{WAM}).
@@ -629,13 +601,13 @@ return(miu.rule)
 #' \item modified center of gravity (\code{COG}).
 #' }
 #' 
-#' @title Defuzzifier to transform from fuzzy terms to crisp values
+#' @title Defuzzifier to transform from linguistic terms to crisp values
 #'
-#' @param data a matrix(m x n) of data, where m is the number of instances and 
-#' n is the number of variables.
+#' @param data a matrix (\eqn{m \times n}) of data, where \eqn{m} is the number of instances and 
+#' \eqn{n} is the number of variables.
 #' @param rule a list or matrix of fuzzy IF-THEN rules, as discussed in \code{\link{rulebase}}.
-#' @param range.output a matrix(2 x n) containing the range of the output data. 
-#' @param names.varoutput a list for giving names to the fuzzy terms. See \code{\link{rulebase}}.
+#' @param range.output a matrix (\eqn{2 \times n}) containing the range of the output data. 
+#' @param names.varoutput a list for giving names to the linguistic terms. See \code{\link{rulebase}}.
 #' @param varout.mf a matrix constructing the membership function of the output variable. 
 #'        See \code{\link{fuzzifier}}.
 #' @param miu.rule the results of the inference module. See \code{\link{inference}}.
@@ -655,10 +627,7 @@ return(miu.rule)
 #' @seealso \code{\link{fuzzifier}}, \code{\link{rulebase}}, and \code{\link{inference}}
 #' @return A matrix of crisp values
 #' @export
-defuzzifier <- function(data, rule, range.output, names.varoutput = NULL, varout.mf = NULL, miu.rule, type.defuz = "MAMDANI", type.model = "TSK", func.tsk = NULL){
-
-## copy rule
-rule.temp <- matrix(unlist(rule), nrow = length(rule), byrow= TRUE)
+defuzzifier <- function(data, rule = NULL, range.output = NULL, names.varoutput = NULL, varout.mf = NULL, miu.rule, type.defuz = NULL, type.model = "TSK", func.tsk = NULL){
 
 ## Inisialitation
 def <- matrix(0, nrow=nrow(data), ncol = 1)
@@ -666,7 +635,9 @@ def.temp <- matrix(0, nrow=nrow(data), ncol = 1)
 
 ## Mamdani type
 if (type.model == 1 || type.model == "MAMDANI"){
-	
+	## copy rule
+	rule.temp <- matrix(unlist(rule), nrow = length(rule), byrow= TRUE)
+
 	## check names.varoutput
 	if (is.null(names.varoutput)){
 		stop("please define the names of output variable")
@@ -684,6 +655,7 @@ if (type.model == 1 || type.model == "MAMDANI"){
 	for (k in 1 : nrow(data)){
 		##Check not zero on miu.rule
 		chck <- which(miu.rule[k, ] != 0)
+		
 		l.chck <- length(chck)
 		cum <- matrix(0, nrow=nrow(data), ncol = l.chck) 
 		div <- matrix(0, nrow=nrow(data), ncol = l.chck)
@@ -704,31 +676,20 @@ if (type.model == 1 || type.model == "MAMDANI"){
 				
 				strg <- c(rule.temp[chck[ii], ncol(rule.temp)])
 				aaa <- which(strg == names.varoutput)
+				
 				if (length(aaa) != 0) {
 				indx[ii, 1] <- aaa
 				indx[ii, 2] <- miu.rule[k, chck[ii]]
 				}
 			}
 			
-			## check duplication on indx, choose with has a max of degree
-			if (nrow(indx) >= 2){
-				for (jj in 1 : (nrow(indx) - 1)){
-					temp.indx[1, ] <- indx[jj, ]
-					for (kk in (jj + 1) : nrow(indx)){
-						ck <- length(which(temp.indx[1, 1] == indx[kk, 1]))
-						if (ck != 0){
-							if (temp.indx[1, 2] >= indx[kk, 2])
-								indx[kk, ] <- NA
-							else 
-								indx[jj, ] <- NA
-						}
-					}
-				}				
+			if (nrow(indx) > 1) {
+				## check duplication on indx, choose with has a max of degree
+				indx <- indx[order(indx[,c(2)], decreasing = TRUE),]
+				indx.nondup <- which(duplicated(indx[, 1]) == FALSE, arr.ind = TRUE)
+				indx <- indx[indx.nondup, ,drop = FALSE] 
 			}
 			
-			# erase NA value on indx
-			indx <- na.omit(indx)
-			
 			#defuzzification procedure for  Centroid
 			if (type.defuz == 1 || type.defuz == "WAM") {								
 				for (i in 1 : nrow(indx)){										
@@ -1100,8 +1061,8 @@ if (type.model == 1 || type.model == "MAMDANI"){
 						average <- (varout.mf[2, indx[i, 1]] + max(varout.mf[2 : 5, indx[i, 1]], na.rm = TRUE)) / 2
 						av.point <- varout.mf[3, indx[i, 1]]
 						# check first which one greater between indx[i, 2] with the value of MF (for centroid)
-						temp1 <- indx[i, 2] * av.point #* abs((average - max(varout.mf[2 : 5, indx[i, 1]], na.rm = TRUE))) / 2
-						temp2 <- indx[i, 2] # * (abs((average - max(varout.mf[2 : 5, indx[i, 1]], na.rm = TRUE))) / 2)
+						temp1 <- indx[i, 2] * av.point 
+						temp2 <- indx[i, 2] 
 						temp <- temp + temp1
 						temp.d <- temp.d + temp2
 					}
@@ -1144,33 +1105,34 @@ else if (type.model == 2 || type.model == "TSK"){
 	}
 		
 	for (k in 1 : nrow(data)){
-		data.k <-(data[k, ])
-		data.m <- as.matrix(data.k)
-		
+		data.m <- data[k, ,drop = FALSE]
+
 		if (ncol(func.tsk) > 1){
-			func.tsk.var <- func.tsk[, 1: ncol(func.tsk) - 1]
-			func.tsk.cont <- t(t(func.tsk[, ncol(func.tsk)]))
+			func.tsk.var <- func.tsk[, -ncol(func.tsk), drop = FALSE]
+			func.tsk.cont <- func.tsk[, ncol(func.tsk), drop = FALSE]
 		
-			ff <- func.tsk.var %*% data.m + func.tsk.cont
+			ff <- func.tsk.var %*% t(data.m) + func.tsk.cont
 		}
 		else if (ncol(func.tsk) == 1){
 			ff <- func.tsk 
 		}
+
+		miu.rule.t <- miu.rule[k, ,drop = FALSE]
 		
-		miu.rule.t <- as.matrix(miu.rule[k, ])
-		cum <- sum(miu.rule.t * ff)	
+		cum <- miu.rule.t %*% ff
 		div <- sum(miu.rule.t)
 		
 		def[k, 1] <- cum / div
+
 		if (div == 0)
 			def[k, 1] <- 0
 		else
 		{
 			def[k, 1] <- cum / div
-			if (def[k, 1] > max(range.output, na.rm=TRUE))
-				def[k, 1] <- max(range.output, na.rm=TRUE)
-			else if (def[k, 1] < min(range.output, na.rm=TRUE))
-				def[k, 1] <- min(range.output, na.rm=TRUE)
+			if (def[k, 1] > max(range.output))
+				def[k, 1] <- max(range.output)
+			else if (def[k, 1] < min(range.output))
+				def[k, 1] <- min(range.output)
 		}
 	}
 }
@@ -1182,7 +1144,7 @@ return(def)
 #' two input variables and one output variable, automatically for all values on a plane. 
 #'
 #' @title A data generator
-#' @param range.input the range of the input variables, as a matrix(2 x n). 
+#' @param range.input the range of the input variables, as a matrix (\eqn{2 \times n}). 
 #' @param num.grid a number representing the size of the grid on the plane.
 #' @return the data
 #' @examples 
@@ -1223,40 +1185,67 @@ for (i in 2 : length.data){
 return(data)
 }
 
-# This function is used to create fuzzy term on each variables
+# This function is used to create linguistic term on each variables
 #
-# @title A fuzzy term creator 
-# @param num.labels a matrix of number of fuzzy labels
-create.fuzzyTerm <- function(num.labels){
-   ## give names for fuzzy values
-	num.inp.var <- sum(num.labels[1, 1 : (ncol(num.labels) - 1)])
-	seq.inp.num <- seq(from = 1, to = num.inp.var, by = 1)
-	temp <- list()
-	k <- 0
-	for (i in 1 : num.inp.var){
-		k <- k + 1
-		j <- ((i - 1) %/% num.labels[1,1]) + 1
-		var <- paste("v", j, sep = ".")
+# @title A linguistic term creator 
+# @param num.labels a matrix of number of linguistic labels
+create.fuzzyTerm <- function(classification = FALSE, num.labels){
+	num.inpvar <- (ncol(num.labels) - 1)
+	if (num.labels[1,1] == 3){
+		fuzzy.labels.inpvar <- rep(c("small", "medium", "large"), num.inpvar)			
+	}
+	else if (num.labels[1,1] == 5){
+		fuzzy.labels.inpvar <- rep(c("very.small", "small", "medium", "large", "very.large"), num.inpvar)		
+	}
+	else if (num.labels[1,1] == 7){
+		fuzzy.labels.inpvar <- rep(c("vv.small", "v.small", "small", "medium", "large", "v.large", "vv.large"), num.inpvar)		
+	}	
+	else {
+		num.inp.var <- sum(num.labels[1, 1 : (ncol(num.labels) - 1)])
+		seq.inp.num <- seq(from = 1, to = num.inp.var, by = 1)
+		temp <- list()
+		k <- 0
+		for (i in 1 : num.inp.var){
+			k <- k + 1
+			j <- ((i - 1) %/% num.labels[1,1]) + 1
+			var <- paste("v", j, sep = ".")
+			
+			fuz <- paste("a", k, sep = ".")				
+			new <- paste(var, fuz, sep ="_")
+			
+			temp <- append(temp, new)
+			if (i %% num.labels[1,1] == 0) {
+				k <- 0
+			}
+		}
 		
-		fuz <- paste("a", k, sep = ".")	
-		new <- paste(var, fuz, sep ="_")
-		
-		temp <- append(temp, new)
-		if (i %% num.labels[1,1] == 0) {
-			k <- 0
+		fuzzy.labels.inpvar <- as.character(temp)
+	
+	}
+	
+	if (classification == TRUE){
+		num.out.var <- num.labels[1, ncol(num.labels)]
+		seq.out.num <- seq(from = 1, to = num.out.var, by = 1)
+		fuzzy.labels.outvar <- paste("c", seq.out.num, sep = ".")
+	}
+	else {
+		if (num.labels[1,ncol(num.labels)] == 3){
+			fuzzy.labels.outvar <- c("small", "medium", "large")
+		}
+		else if (num.labels[1,ncol(num.labels)] == 5){
+			fuzzy.labels.outvar <- c("very.small", "small", "medium", "large", "very.large")		
+		}
+		else if (num.labels[1,ncol(num.labels)] == 7){
+			fuzzy.labels.outvar <- c("vv.small", "v.small", "small", "medium", "large", "v.large", "vv.large")
+		}	
+		else {
+			num.out.var <- num.labels[1, ncol(num.labels)]
+			seq.out.num <- seq(from = 1, to = num.out.var, by = 1)
+			fuzzy.labels.outvar <- paste("c", seq.out.num, sep = ".")
 		}
 	}
 	
-	names.varinput <- as.character(temp)
-	
-	num.out.var <- num.labels[1, ncol(num.labels)]
-	seq.out.num <- seq(from = 1, to = num.out.var, by = 1)
-	names.varoutput <- paste("c", seq.out.num, sep = ".")
-
-	names.variable <- c(names.varinput, names.varoutput)
-	
-	fuzzyTerm <- list(names.fvalinput = names.varinput, names.fvaloutput = names.varoutput)
-	return(fuzzyTerm)
+	return(list(names.fvalinput = fuzzy.labels.inpvar, names.fvaloutput = fuzzy.labels.outvar))
 }
 
 # This function is used to calculate the implication function
@@ -1273,10 +1262,7 @@ calc.implFunc <- function(antecedent, consequent, type.implication.func = "ZADEH
 			temp.rule.degree <- (1 - antecedent)
 	}
 	else if (type.implication.func == "LUKASIEWICZ"){
-		if (consequent < antecedent)
-			temp.rule.degree <- 1 - consequent + antecedent
-		else
-			temp.rule.degree <- 1
+		temp.rule.degree <- min(1 - antecedent + consequent, 1)
 	}
 	else if (type.implication.func == "ZADEH"){
 		if (antecedent < 0.5 || (1 - antecedent) > consequent)
@@ -1323,4 +1309,82 @@ calc.implFunc <- function(antecedent, consequent, type.implication.func = "ZADEH
 		temp.rule.degree <- min(antecedent, consequent)
 	}
 	return (temp.rule.degree)
+}
+
+## It is used to change linguistic term to be unique for inference process
+# @param type.model a type of FRBS model: "MAMDANI", "TSK", etc
+# @param rule a set of fuzzy rules
+# @param varinp.mf a matrix of membership function of input variables
+# @param varout.mf a matrix of membership function of the output variable
+# @param num.varinput the number of input variables
+# @param num.labels a vector of the number of linguistic labels
+ch.unique.fuzz <- function(type.model, rule, varinp.mf, varout.mf = NULL, num.varinput, num.labels){	
+	if (type.model == "MAMDANI"){		
+		num.labels.input <- num.labels[, -ncol(num.labels), drop = FALSE]	
+		if (!is.null(rule) && rule[1, 1] == "IF"){
+			k <- 1
+			new.rule <- matrix(NA, nrow = nrow(rule), ncol = (2 * num.varinput + 1))
+			for (i in 1 : num.varinput) {
+				new.rule[, k] <- paste(rule[, (4 * i), drop = FALSE], i, sep=".")
+				new.rule[, k + 1] <- "and"
+				k <- k + 2
+			}
+			new.rule[, (ncol(new.rule) - 1)] <- "->"
+			new.rule[, ncol(new.rule)] <- paste(rule[, ncol(rule), drop = FALSE], num.varinput + 1, 
+			                              sep=".")
+			rule <- new.rule
+		}
+		else if (!is.null(rule)){
+			i <- 1
+			k <- 1
+			while ( i <= ncol(rule)){
+				rule[, i] <- paste(rule[, i, drop = FALSE], k, sep=".")
+				i <- i + 2
+				k <- k + 1
+			}
+		}
+	}
+	else if (type.model == "TSK") {
+		num.labels.input <- num.labels[, -ncol(num.labels), drop = FALSE]	
+		if (!is.null(rule) && rule[1, 1] == "IF"){
+			k <- 1
+			new.rule <- matrix(NA, nrow = nrow(rule), ncol = (2 * num.varinput))
+			for (i in 1 : num.varinput) {
+				new.rule[, k] <- paste(rule[, (4 * i), drop = FALSE], i, sep=".")
+				new.rule[, k + 1] <- "and"
+				k <- k + 2
+			}
+			new.rule[, ncol(new.rule)] <- "->"
+			rule <- new.rule
+		}
+		else if (!is.null(rule)){
+			i <- 1
+			k <- 1
+			while ( i < ncol(rule)){
+				rule[, i] <- paste(rule[, i, drop = FALSE], k, sep=".")
+				i <- i + 2
+				k <- k + 1
+			}
+		}
+	}	
+	
+	## Change the linguistic values on input data
+	seq.names <- rep(1:num.varinput, num.labels.input)
+	names.fvalinput <- paste(colnames(varinp.mf), seq.names, sep=".")
+	colnames(varinp.mf) <- names.fvalinput
+	## Change the linguistic values on output data
+	if (type.model == "MAMDANI"){
+		seq.names <- rep(num.varinput + 1, num.labels[, ncol(num.labels)])
+		names.fvaloutput <- paste(colnames(varout.mf), seq.names, sep=".")
+		colnames(varout.mf) <- names.fvaloutput
+		names.fvalues <- c(names.fvalinput, names.fvaloutput)
+	}
+	else if (type.model == "TSK"){
+		names.fvaloutput <- colnames(varout.mf)
+		names.fvalues <- c(names.fvalinput, names.fvaloutput)
+	}
+	
+	return (list(rule = rule, varinp.mf = varinp.mf, varout.mf = varout.mf, 
+	           names.fvalinput = names.fvalinput, names.fvaloutput = names.fvaloutput, 
+			   names.fvalues = names.fvalues))
 }
\ No newline at end of file
