@@ -16,44 +16,62 @@
 #  A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 #############################################################################
-#' This is the internal function that implements the model proposed by L.X. Wang and J.M. 
+#' This is the internal function that implements the model proposed by L. X. Wang and J. M. 
 #' Mendel. It is used to solve regression task. Users do not need to call it directly,
 #' but just use \code{\link{frbs.learn}} and \code{\link{predict}}
 #' 
-#' The fuzzy rule-based system for learning from L.X. Wang and J.M. 
-#' Mendel's paper is implemented in this function. For the learning process, there are three stages: 
-#' Step 1 divides the input and output spaces of the given numerical data into fuzzy regions. 
-#' Step 2 generates fuzzy IF-THEN rules from the training data. 
-#' Step 3 determines a degree of each rule. 
-#' In the prediction phase, there are four steps: fuzzification, checking the rules, inference, and defuzzification.
+#' The fuzzy rule-based system for learning from L. X. Wang and J. M. 
+#' Mendel's paper is implemented in this function. For the learning process, there are four stages as follows: 
+#' \itemize{
+#' \item \code{Step 1:} Divide equally the input and output spaces of the given numerical data into 
+#'      fuzzy regions as the database. In this case, fuzzy regions refers to intervals for each 
+#'      linguistic term. Therefore, the length of fuzzy regions represents the number of 
+#'      linguistic terms. For example, the linguistic term "hot" has the fuzzy region \eqn{[1, 3]}. 
+#'      We can construct a triangular membership function having the corner points 
+#'      \eqn{a = 1}, \eqn{b = 2}, and \eqn{c = 3} where \eqn{b} is a middle point 
+#'      that its degree of the membership function equals one. 
+#' \item \code{Step 2:} Generate fuzzy IF-THEN rules covering the training data, 
+#'      using the database from Step 1. First, we calculate degrees of the membership function
+#'      for all values in the training data. For each instance in the training data, 
+#'      we determine a linguistic term having a maximum degree in each variable. 
+#'      Then, we repeat the process for each instance in the training data to construct 
+#'      fuzzy rules covering the training data.
+#' \item \code{Step 3:} Determine a degree for each rule. 
+#'      Degrees of each rule are determined by aggregating the degree of membership functions in 
+#'      the antecedent and consequent parts. In this case, we are using the product aggregation operators. 
+#' \item \code{Step 4:} Obtain a final rule base after deleting redundant rules. 
+#'      Considering degrees of rules, we can delete the redundant rules having lower degrees. 
+#' }
+#' The outcome is a Mamdani model. In the prediction phase, 
+#' there are four steps: fuzzification, checking the rules, inference, and defuzzification.
 #' 
 #' @title WM model building 
 #'
-#' @param data.train a matrix(m x n) of normalized data for the training process, where m is the number of instances and 
-#'        n is the number of variables; the last column is the output variable. 
+#' @param data.train a matrix (\eqn{m \times n}) of normalized data for the training process, where \eqn{m} is the number of instances and 
+#'        \eqn{n} is the number of variables; the last column is the output variable. 
 #'        Note the data must be normalized between 0 and 1. 
-#' @param num.labels a matrix(1 x n), whose elements represent the number of labels (fuzzy terms); 
-#' n is the number of variables.
+#' @param num.labels a matrix (\eqn{1 \times n}), whose elements represent the number of labels (linguistic terms); 
+#' \eqn{n} is the number of variables.
 #' @param type.mf the type of the membership function. See \code{\link{frbs.learn}}.
 #' @param type.tnorm a value which represents the type of t-norm. See \code{\link{inference}}.
-#' @param type.implication.func a value representing type of implication function. Let us consider a rule, a -> b,  
+#' @param type.implication.func a value representing type of implication function. Let us consider a rule, \eqn{a \to b},  
 #' \itemize{
-#' \item \code{DIENES_RESHER} means (b > 1 - a? b : 1 - a).
-#' \item \code{LUKASIEWICZ} means (b < a ? 1 - a + b : 1).
-#' \item \code{ZADEH} means (a < 0.5 || 1 - a > b ? 1 - a : (a < b ? a : b)).
-#' \item \code{GOGUEN} means (a < b ? 1 : b / a).
-#' \item \code{GODEL} means (a <= b ? 1 : b).
-#' \item \code{SHARP} means (a <= b ? 1 : 0).
-#' \item \code{MIZUMOTO} means (1 - a + a * b).
-#' \item \code{DUBOIS_PRADE} means (b == 0 ? 1 - a : (a == 1 ? b : 1)).
-#' \item \code{MIN} means (a < b ? a : b)
+#' \item \code{DIENES_RESHER} means \eqn{(b > 1 - a? b : 1 - a)}.
+#' \item \code{LUKASIEWICZ} means \eqn{(b < a ? 1 - a + b : 1)}.
+#' \item \code{ZADEH} means \eqn{(a < 0.5 || 1 - a > b ? 1 - a : (a < b ? a : b))}.
+#' \item \code{GOGUEN} means \eqn{(a < b ? 1 : b / a)}.
+#' \item \code{GODEL} means \eqn{(a <= b ? 1 : b)}.
+#' \item \code{SHARP} means \eqn{(a <= b ? 1 : 0)}.
+#' \item \code{MIZUMOTO} means \eqn{(1 - a + a * b)}.
+#' \item \code{DUBOIS_PRADE} means \eqn{(b == 0 ? 1 - a : (a == 1 ? b : 1))}.
+#' \item \code{MIN} means \eqn{(a < b ? a : b)}.
 #' }
 #' @param classification a boolean representing whether it is a classification problem or not.
 #' @param range.data a matrix representing interval of data.
 #' @seealso \code{\link{frbs.learn}}, \code{\link{predict}} and \code{\link{frbs.eng}}.
 # @return an object of type \code{frbs}. Please have a look at An \code{\link{frbs-object}} for looking its complete components.
 #' @references 
-#' L. X. Wang and J.M. Mendel, "Generating fuzzy rule by learning from examples", IEEE Trans. Syst., Man, and Cybern.,
+#' L. X. Wang and J. M. Mendel, "Generating fuzzy rule by learning from examples", IEEE Trans. Syst., Man, and Cybern.,
 #' vol. 22, no. 6, pp. 1414 - 1427 (1992).
 # @export
 WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODUCT", type.implication.func = "ZADEH", classification = FALSE, range.data = NULL) {
@@ -65,8 +83,7 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 		range.data <- matrix(nrow = 2, ncol = ncol(data.train))
 		range.data[1, ] <- 0
 		range.data[2, ] <- 1
-	}
-	else {
+	} else {
 		## make range of data according to class on data training
 	    range.data.out <- matrix(c(0.5001, num.labels[1, ncol(num.labels)] + 0.4999), nrow = 2)
 		range.data.inp <- matrix(nrow = 2, ncol = (ncol(data.train) - 1))
@@ -75,15 +92,16 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 		range.data <- cbind(range.data.inp, range.data.out)
 	}
 	
-	## build the names of fuzzy values
-	fuzzyTerm <- create.fuzzyTerm(num.labels)
+	## build the names of linguistic values
+	fuzzyTerm <- create.fuzzyTerm(classification, num.labels)
+	
 	names.fvalinput <- fuzzyTerm$names.fvalinput
 	names.fvaloutput <- fuzzyTerm$names.fvaloutput
 	names.fvalues <- c(names.fvalinput, names.fvaloutput)
 	
 	#get number of row and column
 	nrow.data.train <- nrow(data.train)
-	ncol.data.train <- ncol(data.train)
+	num.varinput <- ncol(data.train)
 	
 	##initialize
 	seg <- matrix(ncol = 1)
@@ -93,10 +111,6 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 	## use data.train as data
 	data <- data.train
 	
-	## get number of input variables as number of column of data
-	num.varinput <- ncol.data.train
-
-
 	####### Wang and Mendel's Steps begin ####################
 	###Step 1: Divide the Input and Output Spaces Into Fuzzy Regions
 	
@@ -104,15 +118,15 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 	jj <- 0
 	
 	## loop for all variable
-	for (i in 1 : ncol.data.train){
+	for (i in 1 : num.varinput){
 		## initialize
 		seg <- num.labels[1, i]
 		kk <- 1
 			
-		## Make the depth on each fuzzy value, assumed it's similar on all region. 
+		## Make the depth on each linguistic value, assumed it's similar on all region. 
 		delta.point <- (range.data[2, i] - range.data[1, i]) / (seg + 1)
 		
-			##contruct matrix of parameter of membership function (var.mf) on each variable (max(var) is ncol.data.train
+			##contruct matrix of parameter of membership function (var.mf) on each variable (max(var) is num.varinput
 			for (j in 1 : num.labels[1, i]){
 			
 				##counter to continue index of column var.mf					
@@ -303,11 +317,9 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 		
 	## get degree of membership by fuzzification
 	MF <- fuzzifier(data, num.varinput, num.labels, var.mf)	
-	
 	colnames(MF) <- c(names.fvalues)
-		
-	####get max value of degree on each variable to get one rule.
 	
+	####get max value of degree on each variable to get one rule.
 	MF.max <- matrix(0, nrow = nrow(MF), ncol = ncol(MF))
 	k <- 1
 	for (i in 1 : length(num.labels)){
@@ -325,7 +337,7 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 	
 	colnames(MF.max) <- c(names.fvalues)
 	rule.matrix <- MF.max
-		
+	
 	## Step III
 	## determine the degree of the rule
 	degree.rule <- matrix(nrow = nrow(rule.matrix), ncol =1)
@@ -367,66 +379,36 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 	rule.matrix[rule.matrix > 0] <- 1
 	rule.matrix.bool <- rule.matrix
 
-	## find the same rows on matrix rule
-	temp <- matrix(nrow = 1, ncol = ncol(rule.matrix.bool))
-	kkk <- 1
+	## find the same elements on matrix rule considering degree of rules
 	rule.red.mat <- rule.matrix
-	degree.rule.temp <- degree.rule
+	temp <- cbind(degree.rule, degree.ante, rule.matrix.bool)
+	temp <- temp[order(temp[,c(1)], decreasing = TRUE),]
+	indx.nondup <- which(duplicated(temp[, -c(1,2)]) == FALSE, arr.ind = TRUE)
+	rule.complete <- temp[indx.nondup, ,drop = FALSE] 
+	degree.ante <- rule.complete[, 2, drop = FALSE] 
+	degree.rule <- rule.complete[, 1, drop = FALSE]
+	rule.red.mat <- rule.complete[, 3:ncol(rule.complete)]
 	
-	for (i in 1 : (nrow(rule.matrix.bool) - 1)){
-		temp.rule <- matrix(rule.matrix.bool[i, ], nrow = 1)
-		ii <- i + 1
-		for (j in ii : nrow(rule.matrix.bool)){
-			sim.check = dist(rbind(temp.rule, matrix(rule.matrix.bool[j, ], nrow = 1)))
-			if (sim.check == 0){
-				if (degree.rule[i] >= degree.rule[j]){
-					rule.red.mat[j, ] <- NA
-					degree.ante[j] <- NA
-					degree.rule.temp[j] <- NA
-				}
-				else {
-					rule.red.mat[i, ] <- NA
-					degree.ante[i] <- NA
-					degree.rule.temp[i] <- NA
-				}
-			}
-		}
-	}	
+	## delete incomplete rule
+	ind.incomplete <- which(rowSums(rule.red.mat) != num.varinput)
+	rule.red.mat[ind.incomplete, ] <- NA	
+	degree.rule[ind.incomplete] <- NA
+	degree.ante[ind.incomplete] <- NA
 	
-	degree.ante <- na.omit(degree.ante)
-	rule.red.mat <- na.omit(rule.red.mat)
-	degree.rule <- na.omit(degree.rule.temp)
+	## create rule in numeric
+	seqq <- seq(1:ncol(rule.red.mat))
+	rule.data.num <- t(apply(rule.red.mat, 1, function(x) x * seqq))
 	
-	## Build Rule database (in list of string) from matrix rule.matrix
-	## rule.data.num is the numbers representing the sequence of string of variable names 	
-	rule.data.num <- matrix(nrow = nrow(rule.red.mat), ncol = ncol(num.labels))	
-	for (i in 1 : nrow(rule.red.mat)){
-		k <- 0
-		for (j in 1 : ncol(rule.red.mat)){
-			if (rule.red.mat[i, j] != 0){
-				k <- k + 1
-				rule.data.num[i, k] <- j 
-				if (k >= ncol(num.labels))
-				break
-			}	
-		}
-	}
-
-	## check is.na on rule.data.num and degree.ante
-	for (i in 1 : nrow(rule.data.num)){
-		if(any(is.na(rule.data.num[i, ])== TRUE)){
-			degree.ante[i, ] <- NA 
-			degree.rule[i, ] <- NA
-		}	
-	}	
-
 	rule.data.num <- na.omit(rule.data.num)
 	degree.ante <- na.omit(degree.ante)
 	degree.rule <- na.omit(degree.rule)
-
+	
+	## rule.data.num is the numbers representing the sequence of string of variable names 	
+	rule.data.num[which(rule.data.num == 0)] <- NA
+	rule.data.num <- t(apply(rule.data.num, 1, na.omit))
 	
 	## build the rule into list of string
-	res <- generate.rule(rule.data.num, num.labels)
+	res <- generate.rule(rule.data.num, num.labels, names.fvalinput, names.fvaloutput)
 	rule <- res$rule
 	
 	#############################################################
@@ -463,25 +445,25 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 #' but just use \code{\link{frbs.learn}} and \code{\link{predict}}. This method is
 #' suitable only for classification problems.
 #' 
-#' This method was proposed by Zheru Chi, Hong Yan, and Tuan Pham that extends
-#' Wang and Mendel's method. The method consists of the following five steps:
-#' \itemize{
-#' \item Step 1: Fuzzify the input space.
-#' \item Step 2: Generate fuzzy rules from given training data pairs.
-#' \item Step 3: Assign a degree to each rule.
-#' \item Step 4: Create a combined rule bank.
-#' \item Step 5: Determine the mapping by using a defuzzification method.
-#' }
+#' This method was proposed by Z. Chi, H. Yan, and T. Pham that extends
+#' Wang and Mendel's method for tackling classification problems. 
+#' Basically, the algorithm is quite similar as Wang and Mendel's technique. 
+#' However, since it is based on the FRBCS model, Chi's method only takes class labels on each data
+#' to be consequent parts of fuzzy IF-THEN rules. In other words, we generate rules as in 
+#' Wang and Mendel's technique (\code{\link{WM}}) and then we replace consequent parts with their classes. 
+#' Regarding calculating degress of each rule, they are determined by antecedent parts of the rules. 
+#' Redudant rules can be deleted by considering their degrees. Lastly, we obtain fuzzy IF-THEN rules 
+#' based on the FRBCS model.
 #'
 #' @title FRBCS.CHI model building 
 #'
-#' @param range.data a matrix (2 x n) containing the range of the normalized data, where n is the number of variables, and
+#' @param range.data a matrix (\eqn{2 \times n}) containing the range of the normalized data, where \eqn{n} is the number of variables, and
 #' first and second rows are the minimum and maximum values, respectively. 
-#' @param data.train a matrix (m x n) of normalized data for the training process, where m is the number of instances and 
-#' n is the number of variables; the last column is the output variable. Note the data must be normalized between 0 and 1. 
-#' @param num.labels a matrix (1 x n), whose elements represent the number of labels (fuzzy terms); 
-#' n is the number of variables.
-#' @param num.class an integer number representing the number of labels (fuzzy terms).
+#' @param data.train a matrix (\eqn{m \times n}) of normalized data for the training process, where \eqn{m} is the number of instances and 
+#' \eqn{n} is the number of variables; the last column is the output variable. Note the data must be normalized between 0 and 1. 
+#' @param num.labels a matrix (\eqn{1 \times n}), whose elements represent the number of labels (linguistic terms); 
+#' \eqn{n} is the number of variables.
+#' @param num.class an integer number representing the number of labels (linguistic terms).
 #' @param type.mf the type of the shape of the membership functions. See \code{\link{fuzzifier}}.
 #' @param type.tnorm the type of t-norm. See \code{\link{inference}}.
 #' @param type.snorm the type of s-norm. See \code{\link{inference}}.
@@ -495,7 +477,7 @@ WM <- function(data.train, num.labels, type.mf = "GAUSSIAN", type.tnorm = "PRODU
 FRBCS.CHI <- function(range.data, data.train, num.labels, num.class, type.mf = "TRIANGLE", type.tnorm = "MIN", type.snorm = "MAX", type.implication.func = "ZADEH") {
 
 	num.labels[1, ncol(num.labels)] <- num.class
-	
+
 	## generate initial model using WM
 	mod <- WM(data.train, num.labels, type.mf, type.tnorm = type.tnorm, type.implication.func = type.implication.func, classification = TRUE)
 	
@@ -503,15 +485,11 @@ FRBCS.CHI <- function(range.data, data.train, num.labels, num.class, type.mf = "
 	rule <- mod$rule
 	rule.constant <- mod$rule
 	degree.ante <- mod$degree.ante
-	 for (i in 1 : nrow(names.fvaloutput)){
-		 for (j in 1 : nrow(rule.constant)){
-			 if (rule[j, ncol(rule)] == names.fvaloutput[i])
-				 rule.constant[j, ncol(rule.constant)] <- i
-		 }
-	 }
 	
-	classes <- matrix(strtoi(rule.constant[, ncol(rule.constant)])) 
+	rule.constant <- cbind(rule.constant[, -ncol(rule.constant), drop = FALSE], 
+	                      as.numeric(factor(rule.constant[, ncol(rule.constant)])))
 	
+	classes <- matrix(strtoi(rule.constant[, ncol(rule.constant)])) 	
 	grade.cert <- cbind(classes, degree.ante)
 	
 	mod$class <- classes
@@ -543,13 +521,13 @@ FRBCS.CHI <- function(range.data, data.train, num.labels, num.class, type.mf = "
 #'
 #' @title FRBCS.W model building 
 #'
-#' @param range.data a matrix (2 x n) containing the range of the normalized data, where n is the number of variables, and
+#' @param range.data a matrix (\eqn{2 \times n}) containing the range of the normalized data, where \eqn{n} is the number of variables, and
 #' first and second rows are the minimum and maximum values, respectively. 
-#' @param data.train a matrix (m x n) of normalized data for the training process, where m is the number of instances and 
-#' n is the number of variables; the last column is the output variable. Note the data must be normalized between 0 and 1. 
-#' @param num.labels a matrix (1 x n), whose elements represent the number of labels (fuzzy terms); 
-#' n is the number of variables.
-#' @param num.class an integer number representing the number of labels (fuzzy terms).
+#' @param data.train a matrix (\eqn{m \times n}) of normalized data for the training process, where \eqn{m} is the number of instances and 
+#' \eqn{n} is the number of variables; the last column is the output variable. Note the data must be normalized between 0 and 1. 
+#' @param num.labels a matrix (\eqn{1 \times n}), whose elements represent the number of labels (linguistic terms); 
+#' \eqn{n} is the number of variables.
+#' @param num.class an integer number representing the number of labels (linguistic terms).
 #' @param type.mf the type of the shape of the membership functions.
 #' @param type.tnorm the type of t-norm. See \code{\link{inference}}.
 #' @param type.snorm the type of s-norm. See \code{\link{inference}}.
@@ -565,20 +543,19 @@ FRBCS.W <- function(range.data, data.train, num.labels, num.class, type.mf, type
 	num.labels[1, ncol(num.labels)] <- num.class
 	
 	## generate initial model using WM
-	mod <- WM(data.train, num.labels, type.mf, type.tnorm = type.tnorm, type.implication.func = type.implication.func, classification = TRUE)
+	mod <- WM(data.train, num.labels, type.mf, type.tnorm = type.tnorm, type.implication.func = type.implication.func, 
+	          classification = TRUE)
 	
 	names.fvaloutput <- as.matrix(colnames(mod$varout.mf))
 	rule <- mod$rule
 	rule.constant <- mod$rule
 	degree.ante <- mod$degree.ante
-	 for (i in 1 : nrow(names.fvaloutput)){
-		 for (j in 1 : nrow(rule.constant)){
-			 if (rule[j, ncol(rule)] == names.fvaloutput[i])
-				 rule.constant[j, ncol(rule.constant)] <- i
-		 }
-	 }
+	
+	rule.constant <- cbind(rule.constant[, -ncol(rule.constant), drop = FALSE], 
+	                      as.numeric(factor(rule.constant[, ncol(rule.constant)])))
 	
 	classes <- matrix(strtoi(rule.constant[, ncol(rule.constant)])) 
+	
 	grade.cert.temp <- cbind(classes, degree.ante)
 	class.fact <- factor(grade.cert.temp[, 1], exclude = "")
 	beta.class <- as.double(as.matrix(aggregate(grade.cert.temp[, 2], by = list(class.fact), sum)))
